#GMOuse.py

import pygame
import time
from code.misc import *
from code.mouse import Mouse
from code.maze import Maze

inventorySize = 7 #takes up one row of top left corner
inventorySpacing = 20 #in pixels
stillTimeOut = 5 #seconds

mutations = ["", "", "Franken", "", "glo", "spidey", "mer", "homing", "mini"]
activeItem = None
text = []
for i in range(numItems):
	text.append(None)

#setup levels:
#add new levels here:
import code.maze1
import code.maze2
import code.maze3
import code.maze4
import code.maze5
levels = [code.maze3, code.maze4, code.maze1, code.maze2, code.maze5]

currentLevel = 0
maze = None
			

#init pygame:
screen = pygame.display.set_mode((w*tileSize, h*tileSize))
darkScreen = pygame.surface.Surface((w*tileSize, h*tileSize), pygame.SRCALPHA)

#init tile images:
for i in range(numTiles):
	tileImages.append(pygame.image.load("img/tile%s.png" % (i + 1,)).convert())
	
#init item images
for i in range(numItems):
	itemImages.append(pygame.image.load("img/item%s.png" % (i + 1,)))
		
def drawInventory():
	for i in range(len(game.inventory)):
		item = game.inventory[i]
		if item:
			if i < inventorySize:
				#row 1, right to left
				screen.blit(itemImages[item], (tileSize * w - (tileSize + inventorySpacing) * (i + 1), 0))
				if text[item]:
					screen.blit(text[item], (tileSize * w - (tileSize + inventorySpacing) * (i + 1), 45))
				
def drawCursor():
	if activeItem:
		#potions begin at item 2
		pos = pygame.mouse.get_pos()
		screen.blit(pygame.image.load("img/cursor" + str(activeItem - 1) + ".png"), pos)
		
		
helpText = [
"Free the mice and bring them back to the start zone.",
"Mutate mice to get past obstacles.",
"Left click: toggle walls, select inventory items",
"Right click: cancel inventory item",
"H: open/close help",
"P: pause",
"R: restart level"
]
helpImages = []
y = 20
help = pygame.surface.Surface((400,250))
help.fill((40,40,100))
helpPos = (w * tileSize / 2 - 400 / 2, h * tileSize / 2 - 250 / 2)
pygame.draw.rect(help, (255,255,200), (2,2,396,246), 4)
for line in helpText:
	image = (pygame.font.SysFont("comic_sans", 16).render(line, 1, (255,255,200)))
	help.blit(image, (20,y))
	y += 30
	
def drawHelp():
	screen.blit(help, helpPos)

def drawInfo():
	global maze, runningScore, game
	targetMiceText = (pygame.font.SysFont("comic_sans", 16).render("Mice to win: " + str(maze.targetMice), 1, (0,0,0)))
	cagedMiceText = (pygame.font.SysFont("comic_sans", 16).render("Cages remaining: " + str(maze.cages), 1, (0,0,0)))
	runningScoreText = (pygame.font.SysFont("comic_sans", 16).render("Score: " + str(int(sum(runningScore))), 1, (0,0,0)))
	levelScoreText = (pygame.font.SysFont("comic_sans", 16).render("Level score: " + str(int(game.levelScore)), 1, (0,0,0)))
	screen.blit(targetMiceText, (w * tileSize - 8 * (tileSize + inventorySpacing), int(3 * tileSize / 2)))
	screen.blit(cagedMiceText, (w * tileSize - 6 * (tileSize + inventorySpacing), int(3 * tileSize / 2)))
	screen.blit(runningScoreText, (w * tileSize - 4 * (tileSize + inventorySpacing), int(3 * tileSize / 2)))
	screen.blit(levelScoreText, (w * tileSize - 4 * (tileSize + inventorySpacing), int(7 * tileSize / 4)))
	
#the input function:
def handleInput():
	global activeItem, game, running, paused, helpOn, dt, text, currentLevel
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			running = 0
		elif event.type == pygame.MOUSEBUTTONDOWN:
			if event.button == 1:
				y = (event.pos[1]) / (tileSize)
				x = (event.pos[0]) / (tileSize)
				invX = (event.pos[0]) / (tileSize + inventorySpacing)
				invW = w * tileSize / (tileSize + inventorySpacing) #currently 14
				if invX > invW - (inventorySize) and y < 2:
					#this is in the inventory.
					slot = invW - invX - 1
					if slot < len(game.inventory):
						activeItem = game.inventory[slot]
				elif activeItem:
					#try mutating a mouse!
					for mouse in game.mice:
						if -1 < mouse.pos[0] - x < 1 and -1 < mouse.pos[1] - y < 2:
							success = mouse.mutate(mutations[activeItem])
							text[activeItem] = pygame.font.SysFont("comic_sans", 12).render("%smouse"%(mouse.mutation), 1, (0,0,100))
							if success:
								break
				else:
					i = (event.pos[1] + tileSize / 4) * 2 / tileSize
					if i % 2:
						j = (event.pos[0]+ tileSize / 2) / tileSize
					else: 
						j = event.pos[0] / tileSize
					if maze.walls[i][j] == 2:
						maze.walls[i][j] = 0
					elif maze.walls[i][j] == 0:
						maze.walls[i][j] = 2
			elif event.button == 3:
				activeItem = None
		elif event.type == pygame.KEYDOWN:
			if event.key == pygame.K_h or event.key == pygame.K_QUESTION or event.key == pygame.K_F1:
				if helpOn:
					helpOn = 0
					paused = 0
				else:
					helpOn = 1
					paused = 1
			elif event.key == pygame.K_p or event.key == pygame.K_SPACE:
				if paused:
					paused = 0
				else:
					paused = 1
			elif event.key == pygame.K_r:
				game, running, paused, helpOn, dt = startGame()
			elif event.key == pygame.K_F9 or event.key == pygame.K_9:
				game.levelScore -= 500
				game.mice.append(Mouse(maze, game, 0, 0, 180))
			elif event.key == pygame.K_F10 or event.key == pygame.K_0:
				currentLevel += 1
				currentLevel %= len(levels)				
				game, running, paused, helpOn, dt = startGame()

def startGame():
	global maze
	reload(levels[currentLevel])
	maze = levels[currentLevel].maze
	game = Game()
	game.mice.append(Mouse(maze, game, 0, 0, 180))
	running = 1
	paused = 0
	helpOn = 0
	dt = 0
	return game, running, paused, helpOn, dt
					
class Game:
	def __init__(self):
		self.inventory = []
		self.mice = []
		self.levelScore = 50
	
runningScore = []
def run():
	global game, running, paused, helpOn, dt, runningScore
	pygame.init()
	game, running, paused, helpOn, dt = startGame()
	for i in range(len(levels)):
		runningScore.append(0)
	clock = pygame.time.Clock()
	stillTimer = 0
	#the main loop:
	while running:
		if paused:
			dt = 0
		game.levelScore -= dt
		handleInput()
		maze.updateDarkness()
		maze.drawMaze(screen, darkScreen)
		for mouse in game.mice:
			mouse.mouselightUpdate()
		moving = False
		for mouse in game.mice:
			moving = mouse.updateMouse(dt) or moving
			mouse.drawMouse(screen, darkScreen)
		screen.blit(darkScreen, (0,0))
		drawInventory()
		drawInfo()
		drawCursor()
		if helpOn:
			drawHelp()
		pygame.display.flip()
		dt = min(clock.tick(60) / 1000., .1)
		if not moving:
			stillTimer += dt
			if stillTimer > stillTimeOut:
				#win or reset.
				if maze.targetMice <= 0:
					game.levelScore += 500
					runningScore[currentLevel] = game.levelScore
					currentLevel += 1
					currentLevel %= len(levels)
					sounds['laugh'].play()
				else:
					sounds['ohno'].play()
				game, running, paused, helpOn, dt = startGame()
		else:
			stillTimer = 0
		
		
if __name__ == "__main__":
	run()
	
	
	
	
	
	
	
	