#mouse.py

from misc import *
import pygame
import random

gloRadius = 2

class Mouse:
	images = [pygame.image.load("img/mouse3.png"), pygame.image.load("img/mouse4.png")]
	dir = 0
	pos = [0,0]
	goal = [-1,-1]
	mutation = ""
	mutant = False
	timeOffset = 0
	timePerFrame = .15 #seconds per frame
	thresh = .1
	def __init__(self, maze, game, x=0, y=0, startdir=0):
		self.timeOffset = 0
		self.pos = [x,y]
		self.dir = startdir
		self.currentFrame = 0
		self.state = "running"
		self.handedness = random.randint(0,1) #True for left, false for right
		self.mutation = ""
		self.mutant = False
		self.images = [pygame.image.load("img/mouse3.png"), pygame.image.load("img/mouse4.png")]
		self.maze = maze
		self.game = game
		sounds['eek2'].play()
	
	def updateMouse(self, dt):
		self.timeOffset += dt
		
		#dead mouse does nothing
		if self.state == "dead":
			return
		
		if self.thresholdCheck():
			if self.state == "catapulted":
				#check if current tile is goal, run if it is
				if int(self.pos[0]) == self.goal[0] and int(self.pos[1]) == self.goal[1]:
					self.goal = [-1,-1]
					self.startMouse()
			if self.state == "running":
				self.tileCheck()
				self.itemCheck()
				self.wallCheck()
				#check if current tile is goal, stop if it is
				#this is not in tileCheck method because of return, could be streamined
				if int(self.pos[0]) == self.goal[0] and int(self.pos[1]) == self.goal[1] and self.state != "catapulted":
					self.stopMouse()
					return
			elif self.state == "stopped":
				#check if current tile is goal, stop if it is
				if int(self.pos[0]) == self.goal[0] and int(self.pos[1]) == self.goal[1]:
					self.stopMouse()
				else:
					self.wallCheck()
		
		if self.state == "running" or self.state == "catapulted":
			#move forward:
			self.pos[0] += cos(self.dir) * dt * tileSize / 4 * self.timePerFrame
			self.pos[1] += sin(self.dir) * dt * tileSize / 4 * self.timePerFrame
		#advance animation:
		if self.timeOffset >= self.timePerFrame:
			self.currentFrame += 1
			self.currentFrame %= len(self.images)
			self.timeOffset -= self.timePerFrame
		return self.state == "running" or self.state == "catapulted"
			
	def mouselightUpdate(self):
		#glomouse glow
		if self.mutation == "glo":
			for x in range(int(self.pos[0] - gloRadius), int(self.pos[0] + gloRadius) + 1):
				for y in range(int(self.pos[1] - gloRadius), int(self.pos[1] + gloRadius) + 1):
					if 0<=x<w and 0<=y<h:
						if self.maze.darkness[y][x] > 0:
							self.maze.darkness[y][x] = -1
			
	def thresholdCheck(self):
		if (self.dir == 0 or self.dir == 180):
			if (self.pos[1] - int(self.pos[1]) < self.thresh):
				return True
		elif (self.dir == 90 or self.dir == 270):
			if (self.pos[0] - int(self.pos[0]) < self.thresh):
				return True
				
	def wallCheck(self):
		wall = True
		darkness = 1
		numdir = 1
		
		if self.state == "catapulted":
			return
		
		#turn if see wall:
		while ((wall or darkness > 0) and numdir < 5):
			row, col = findWall(int(self.pos[0]), int(self.pos[1]), self.dir)
			wall = self.maze.walls[row][col]
			
			if self.dir == 0:
				darkRow, darkCol = [int(self.pos[1]) - 1,int(self.pos[0])]
			elif self.dir == 90:
				darkRow, darkCol = [int(self.pos[1]),int(self.pos[0]) + 1]
			elif self.dir == 180:
				darkRow, darkCol = [int(self.pos[1]) + 1,int(self.pos[0])]
			elif self.dir == 270:
				darkRow, darkCol = [int(self.pos[1]),int(self.pos[0]) - 1]	
			darkness = self.maze.darkness[darkRow][darkCol]
			
			#frankenmouse eats walls
			if self.mutation == "Franken" and wall == 3:
				self.maze.walls[row][col] = 0
				sounds["saw"].play()
				wall = False
				
			#glomice walk in the dark
			if self.mutation == "glo":
				darkness = -1
			
			#try straight, left, right, back, in that order
			if (wall or darkness > 0):
				#right-handed mouses go right
				if self.handedness:
					self.dir += numdir * 90
				else:
					self.dir -= numdir * 90
				self.dir %= 360
				numdir += 1

		if numdir >= 5:
			self.stopMouse()
		else:
			self.startMouse()
				
	def tileCheck(self):
		#check for tile effects
		currenttile = self.maze.tiles[int(self.pos[1])][int(self.pos[0])]
		
		#normal tiles
		if not (2 <= currenttile <= 15) and self.mutation == "mer" and self.mutant:
			#mermouse is amphibious
			self.images = [pygame.image.load("img/mouse3.png"), pygame.image.load("img/mouse4.png")]
			self.mutant = False
		#water tiles
		elif 2 <= currenttile <= 15 and self.mutation == "mer" and not self.mutant:
				#mermouse is amphibious
				self.images = [pygame.image.load("img/mermouse3.png"), pygame.image.load("img/mermouse4.png")]
				sounds['splash'].play()
				self.mutant = True
		elif 2 <= currenttile <= 15 and self.mutation == "mini":
			self.killMouse()
			sounds['blub'].play()
		elif (currenttile == 2 or 11 <= currenttile <= 14) and not self.mutation == "mer":
			self.killMouse()
			sounds['blub'].play()
		#mousepad tiles
		elif 20 <= currenttile <= 25:
			emptyTile = True
			for mouse in self.game.mice:
				if mouse != self:
					if int(mouse.pos[0]) == int(self.pos[0]) and int( mouse.pos[1]) == int(self.pos[1]):
						emptyTile = False
			if emptyTile:
				self.goal = [int(self.pos[0]), int(self.pos[1])]
				#get points
				self.maze.targetMice -= 1
				self.game.levelScore += 50
					
		#hole tiles
		elif 16 <= currenttile <= 19:
			#if there's a web, walk over it
			if self.maze.items[int(self.pos[1])][int(self.pos[0])] == 17:
				return
			elif self.mutation == "spidey":
				#set goal to here to stop and build web
				self.goal = [int(self.pos[0]),int(self.pos[1])]
			else:
				self.killMouse()

	def itemCheck(self):
		#check for item effects
		currenttile = self.maze.items[int(self.pos[1])][int(self.pos[0])]
		
		#cage
		if currenttile == 1:
			#empty cage of mouse
			self.maze.items[int(self.pos[1])][int(self.pos[0])] = 0
			self.game.mice.append(Mouse(self.maze, self.game, self.pos[0], self.pos[1], (self.dir + 180) % 360))
			#get points
			self.game.levelScore += 50
		
		if 2 <= currenttile <= 8:
			#acquire mutation
			self.game.inventory.append(currenttile)
			self.maze.items[int(self.pos[1])][int(self.pos[0])] = 0
			sounds["mutagen"].play()
		
		#mousetrap
		if 9 <= currenttile <= 12:
			#set off trap
			sounds['mousetrap'].play()
			if currenttile == 9:
				self.dir = 0
			elif currenttile == 10:
				self.dir = 90
			elif currenttile == 11:
				self.dir = 180
			elif currenttile == 12:
				self.dir = 270
			self.maze.items[int(self.pos[1])][int(self.pos[0])] += 4
			if self.mutation == "mini":
				self.dir = (self.dir + 180) % 360
				self.catapultMouse()
				sounds['trill'].play()
			else:
				self.killMouse()		
		
	def stopMouse(self):
		if self.state != "stopped":
			#mermouse uses mouse's stopped images when not mutant
			if not self.mutant:
				name = "mouse"
			else:
				name = self.mutation + "mouse"
			#reset image to stopped images
			self.images = [pygame.image.load("img/" + name + "1.png"), pygame.image.load("img/" + name + "2.png")]
			self.state = "stopped"
			#spideymouse builds a web
			if self.mutation == "spidey":
				self.maze.items[int(self.pos[1])][int(self.pos[0])] = 17
		
	def startMouse(self):
		#no reanimation plz thx
		if self.state != "running" and self.state != "dead":
			#mermouse uses mouse's running images when not mutant
			if not self.mutant:
				name = "mouse"
			else:
				name = self.mutation + "mouse"
			#reset image to running images
			self.images = [pygame.image.load("img/" + name + "3.png"), pygame.image.load("img/" + name + "4.png")]
			self.state = "running"
			
	def catapultMouse(self):
		self.state = "catapulted"
		if not self.mutant:
			name = "mouse"
		else:
			name = self.mutation + "mouse"
		self.images = [pygame.image.load("img/" + name + "3.png"), pygame.transform.flip(pygame.image.load("img/" + name + "3.png"),True, False)]
		i = 0
		posx = int(self.pos[0] + 3 * cos(self.dir))
		posy = int(self.pos[1] + 3 * sin(self.dir))
		self.goal = [int(posx), int(posy)]
		#look for walls to stop at, don't go more than 3 tiles
		while i < 3:
			row = 0
			col = 0
			posx -= cos(self.dir)
			posy -= sin(self.dir)
			row, col = findWall(posx, posy, self.dir)
			if self.maze.walls[row][col]:
				self.goal = [posx, posy]
			i += 1

	def killMouse(self):
		name = self.mutation + "mouse"
		#reset image to dead image
		self.images = [pygame.image.load("img/" + name + "5.png"), pygame.image.load("img/" + name + "5.png")]
		self.state = "dead"
		sounds['eek'].play()
		
	def mutate(self, mutation):
		#mutant mice are not mutable
		ret = False
		if not self.mutation or self.mutation == "mer":
			if not self.mutation:
				self.mutation = mutation
				ret = True
				sounds[mutation].play()
			self.mutant = True
			name = self.mutation + "mouse"
			#maintain state
			if self.state == "dead":
				self.images = [pygame.image.load("img/" + name + "5.png"), pygame.image.load("img/" + name + "5.png")]
			elif self.state == "stopped":
				self.images = [pygame.image.load("img/" + name + "1.png"), pygame.image.load("img/" + name + "2.png")]
			elif self.state == "running":
				self.images = [pygame.image.load("img/" + name + "3.png"), pygame.image.load("img/" + name + "4.png")]
		return ret

			
	def drawMouse(self, screen, darkScreen):
		#rotational states
		if self.state == "running" or self.state == "dead" or self.state == "catapulted":
			screen.blit(pygame.transform.rotate(self.images[self.currentFrame], -self.dir), 
						(self.pos[0] * tileSize, self.pos[1] * tileSize))
		#non-rotational states
		elif self.state == "stopped":
			screen.blit(self.images[self.currentFrame], (self.pos[0] * tileSize, self.pos[1] * tileSize))
		#glomouse glo
		if self.mutation == "glo":
			pygame.draw.circle(darkScreen, (250,250,100,50), (int(self.pos[0] * tileSize + tileSize/2), int(self.pos[1] * tileSize + tileSize/2)), int(tileSize * (gloRadius + .8)))
			
			
#is there a wall in front of the mouse?
def findWall(posx, posy, dir):
	#get the odd row for horizontal:
	if dir == 0:
		row = (posy + 1) * 2 - 2
		col = posx
	if dir == 180:
		row = (posy + 1) * 2
		col = posx
	#get the even row for vertical walls:
	if dir == 90:
		row = (posy + 1) * 2 - 1
		col = posx + 1
	if dir == 270:
		row = (posy + 1) * 2 - 1
		col = posx
	return row, col